from collections import OrderedDict
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType, SetResponseType


class Pb(ApiGroup):
    def get_pb_match(self, phone_number: str) -> GetResponseType:
        """
        Find number in PhoneBook
        :param phone_number:
        :return:
        """
        return self._connection.post_get('pb/pb-match', {
            'Phone': phone_number
        })

    def get_pb_list(self,
                    page: int = 1,
                    key_word: str='',
                    group_id: int=0,
                    read_count: int=50,
                    save_type: int=0,
                    sort_type: int=1,
                    ascending: int=1
                    ) -> GetResponseType:
        return self._connection.post_get('pb/pb-list', OrderedDict((
            ('GroupID', group_id),
            ('PageIndex', page),
            ('ReadCount', read_count),
            ('SaveType', save_type),
            ('SortType', sort_type),
            ('Ascending', ascending), 
            ('KeyWord', key_word)
        )))

    def pb_count(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._connection.post_get('pb/pb-count')

    def group_count(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._connection.post_get('pb/group-count')


    def pb_new( self, 
                group_id: int=0,
                save_type: int=0,
                name: str='', 
                mobile_phone: str='', 
                home_phone: str='', 
                work_phone: str='', 
                work_email: str='') -> SetResponseType:
        """
        Add new entry to global PhoneBook
        :param
        :return:
        """
        class Node:
            def __init__(self, name):
                self._name = name

            def __str__(self):
                return self._name

        data = OrderedDict([
            ('GroupID',  group_id),
            ('SaveType', save_type),
            (Node('Field'), {
                'Name': 'FormattedName',
                'Value': name
            }),
            (Node('Field'), {
                'Name': 'MobilePhone',
                'Value': mobile_phone
            }),
            (Node('Field'), {
                'Name': 'HomePhone',
                'Value': home_phone
            }),
            (Node('Field'), {
                'Name': 'WorkPhone',
                'Value': work_phone
            }),
            (Node('Field'), {
                'Name': 'WorkEmail',
                'Value': work_email
            }),
        ])

        dicttoxml_xargs = {
            'attr_type': False
        }

        return self._connection.post_set('pb/pb-new', data, dicttoxml_xargs=dicttoxml_xargs)

    def pb_delete(self, pb_index: int) -> SetResponseType:
        """
        Delete single PB Item by its ID
        :param pb_index: Id of PB Item you wish to delete
        :return: dict
        """
        return self._connection.post_set('pb/pb-delete', {'Index': pb_index})

    def group_delete(self, group_id: int) -> SetResponseType:
        """
        Delete Phonebook group by its ID
        :param group_id: Id of Group you wish to delete
        :return:
        """
        return self._connection.post_set('pb/group-delete', {'GroupID': group_id})

    def group_list(self,
                    page: int = 1,
                    read_count: int=50,
                    sort_type: int=1,
                    ascending: int=1
                    ) -> GetResponseType:
        return self._connection.post_get('pb/group-list', OrderedDict((
            ('PageIndex', page),
            ('ReadCount', read_count),
            ('SortType', sort_type),
            ('Ascending', ascending)
        )))

    def group_new(self, name_str: str) -> SetResponseType:
        """
        Create new Phonebook group by its name
        :param name_str: name of PB Group you wish to delete
        :return:
        """
        return self._connection.post_set('pb/group-new', {'GroupName': name_str})

# missing functions
#
# pb/group-update
# pb/pb-copySIM
# pb/pb-export
# pb/pb-import
# pb/pb-import-field
# pb/pb-import-length
# pb/pb-import-match
# pb/pb-update
# pb/pb-move
# pb/pb-match



from collections import OrderedDict
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType, SetResponseType


class Net(ApiGroup):
    def current_plmn(self) -> GetResponseType:
        return self._connection.get('net/current-plmn')

    def net_mode(self) -> GetResponseType:
        return self._connection.get('net/net-mode')

    def set_net_mode(self, lteband: str, networkband: str, networkmode: str) -> SetResponseType:
        return self._connection.post_set('net/net-mode', OrderedDict((
            ('NetworkMode', networkmode),
            ('NetworkBand', networkband),
            ('LTEBand', lteband)
        )))

    def network(self) -> GetResponseType:
        return self._connection.get('net/network')

    def register(self) -> GetResponseType:
        return self._connection.get('net/register')

    def set_register(self, mode: str, plmn: str, rat: str) -> SetResponseType:
        """
        Sets network selection
        :param mode: "1": manual network selection, "0": auto
        :param plmn: Plmn code ("Numeric" value returned by net_mode_list()),
            "" for auto
        :param rat: "0": "2G", "2": "3G", "7": "4G" ("Rat" value returned by
            net_mode_list()), "" for auto
        :return: str
        """
        return self._connection.post_set('net/register', OrderedDict((
            ('Mode', mode),
            ('Plmn', plmn),
            ('Rat', rat)
        )))

    def net_mode_list(self) -> GetResponseType:
        return self._connection.get('net/net-mode-list')

    def plmn_list(self) -> GetResponseType:
        """
    !FIXME LOL DoS! no auth required :-D
    :return:
    """
        return self._connection.get('net/plmn-list')

    def net_feature_switch(self) -> GetResponseType:
        return self._connection.get('net/net-feature-switch')

    def cell_info(self) -> GetResponseType:
        return self._connection.get('net/cell-info')

    def csps_state(self) -> GetResponseType:
        return self._connection.get('net/csps_state')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Cradle(ApiGroup):
    def status_info(self) -> GetResponseType:
        return self._connection.get('cradle/status-info')

    def feature_switch(self) -> GetResponseType:
        return self._connection.get('cradle/feature-switch')

    def basic_info(self) -> GetResponseType:
        return self._connection.get('cradle/basic-info')

    def factory_mac(self) -> GetResponseType:
        return self._connection.get('cradle/factory-mac')

    def mac_info(self) -> GetResponseType:
        return self._connection.get('cradle/mac-info')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class VSim(ApiGroup):
    def operateswitch_vsim(self) -> GetResponseType:
        return self._connection.get('vsim/operateswitch-vsim')



from collections import OrderedDict
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType, SetResponseType


class Security(ApiGroup):

    def bridgemode(self) -> GetResponseType:
        return self._connection.get('security/bridgemode')

    def get_firewall_switch(self) -> GetResponseType:
        return self._connection.get('security/firewall-switch')

    def set_firewall_switch(self,
                            firewall: bool=True,
                            ip_filter: bool=False,
                            wan_ping_filter: bool=True,
                            url_filter: bool=False,
                            mac_filter: bool=False
                            ) -> SetResponseType:
        return self._connection.post_set('security/firewall-switch', OrderedDict((
            ('FirewallMainSwitch', int(firewall)),
            ('FirewallIPFilterSwitch', int(ip_filter)),
            ('FirewallWanPortPingSwitch', int(wan_ping_filter)),
            ('firewallurlfilterswitch', int(url_filter)),
            ('firewallmacfilterswitch', int(mac_filter))
        )))

    def mac_filter(self) -> GetResponseType:
        return self._connection.get('security/mac-filter')

    def lan_ip_filter(self) -> GetResponseType:
        return self._connection.get('security/lan-ip-filter')

    def virtual_servers(self) -> GetResponseType:
        return self._connection.get('security/virtual-servers')

    def url_filter(self) -> GetResponseType:
        return self._connection.get('security/url-filter')

    def upnp(self) -> GetResponseType:
        return self._connection.get('security/upnp')

    def set_upnp(self, enabled: bool) -> SetResponseType:
        return self._connection.post_set('security/upnp', {
            'UpnpStatus': int(enabled),
        })

    def dmz(self) -> GetResponseType:
        return self._connection.get('security/dmz')

    def set_dmz(self, enabled: bool, ip_address: str) -> SetResponseType:
        return self._connection.post_set('security/dmz', OrderedDict((
            ('DmzStatus', int(enabled)),
            ('DmzIPAddress', ip_address)
        )))

    def sip(self) -> GetResponseType:
        return self._connection.get('security/sip')

    def set_sip(self, enabled: bool, port: int) -> SetResponseType:
        return self._connection.post_set('security/sip', OrderedDict((
            ('SipStatus', int(enabled)),
            ('SipPort', port)
        )))

    def feature_switch(self) -> GetResponseType:
        return self._connection.get('security/feature-switch')

    def nat(self) -> GetResponseType:
        return self._connection.get('security/nat')

    def special_applications(self) -> GetResponseType:
        return self._connection.get('security/special-applications')

    def white_lan_ip_filter(self) -> GetResponseType:
        return self._connection.get('security/white-lan-ip-filter')

    def white_url_filter(self) -> GetResponseType:
        return self._connection.get('security/white-url-filter')

    def acls(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage, probably not implemented by Huawei
        :return:
        """
        return self._connection.get('security/acls')



import logging
import re
from typing import Any, Callable, Dict, List, Optional, Tuple, TypeVar, Union, cast
import urllib.parse
import dicttoxml
import xmltodict
import requests
from huawei_lte_api.enums.client import ResponseCodeEnum
from huawei_lte_api.exceptions import \
    ResponseErrorException, \
    ResponseErrorLoginRequiredException, \
    ResponseErrorNotSupportedException, \
    ResponseErrorSystemBusyException, \
    ResponseErrorLoginCsrfException


_LOGGER = logging.getLogger(__name__)

T = TypeVar("T")
GetResponseType = Dict[str, Any]
SetResponseType = str


def _try_or_reload_and_retry(fn: Callable[..., T]) -> Callable[..., T]:
    def wrapped(*args: Any, **kw: Any) -> T:
        try:
            return fn(*args, **kw)
        except ResponseErrorLoginCsrfException:
            args[0].reload()
            return fn(*args, **kw)

    return wrapped


class Connection:
    csrf_re = re.compile(r'name="csrf_token"\s+content="(\S+)"')
    request_verification_tokens = []  # type: List[str]

    def __init__(self, url: str, timeout: Union[float, Tuple[float, float], None] = None):
        self.session = requests.Session()
        self.url = url
        self.timeout = timeout
        if not self.url.endswith('/'):
            self.url += '/'
        self._initialize_csrf_tokens_and_session()

    def reload(self) -> None:
        self._initialize_csrf_tokens_and_session()

    @staticmethod
    def _create_request_xml(data: Union[dict, list], dicttoxml_xargs: Optional[dict]=None) -> bytes:
        if not dicttoxml_xargs:
            dicttoxml_xargs = {}

        dicttoxml_xargs['attr_type'] = False
        return dicttoxml.dicttoxml(data, custom_root='request', **dicttoxml_xargs)

    @staticmethod
    def _process_response_xml(response: requests.Response) -> dict:
        # In some cases unsupported methods, e.g. in config namespace,
        # respond with a redirect to the home page, which may not
        # parse as XML (even though it's labeled XHTML). Try to detect
        # such cases, and return a generated "not supported" error
        # instead of letting the XML parse error pass through.
        xml = response.content
        try:
            return xmltodict.parse(xml, dict_constructor=dict) if xml else {}
        except:
            if response.history:
                return dict(error=dict(code=ResponseCodeEnum.ERROR_SYSTEM_NO_SUPPORT, message=''))
            raise

    @staticmethod
    def _check_response_status(data: dict) -> Union[dict, str]:
        error_code_to_message = {
            ResponseCodeEnum.ERROR_SYSTEM_BUSY.value: 'System busy',
            ResponseCodeEnum.ERROR_SYSTEM_NO_RIGHTS.value: 'No rights (needs login)',
            ResponseCodeEnum.ERROR_SYSTEM_NO_SUPPORT.value: 'No support',
            ResponseCodeEnum.ERROR_SYSTEM_UNKNOWN.value: 'Unknown',
            ResponseCodeEnum.ERROR_SYSTEM_CSRF.value: 'Session error'
        }

        error_code_to_exception = {
            ResponseCodeEnum.ERROR_SYSTEM_BUSY.value: ResponseErrorSystemBusyException,
            ResponseCodeEnum.ERROR_SYSTEM_NO_RIGHTS.value: ResponseErrorLoginRequiredException,
            ResponseCodeEnum.ERROR_SYSTEM_NO_SUPPORT.value: ResponseErrorNotSupportedException,
            ResponseCodeEnum.ERROR_SYSTEM_UNKNOWN.value:  ResponseErrorException,
            ResponseCodeEnum.ERROR_SYSTEM_CSRF.value: ResponseErrorLoginCsrfException
        }
        if 'error' in data:
            error_code = int(data['error']['code'])
            if not data['error']['message']:
                message = error_code_to_message.get(error_code, 'Unknown')
            else:
                message = data['error']['message']
            raise error_code_to_exception.get(error_code, ResponseErrorException)(
                '{}: {}'.format(error_code, message),
                error_code
            )

        response = data['response'] if 'response' in data else data
        return response if response is not None else {}

    def _initialize_csrf_tokens_and_session(self) -> None:
        # Reset
        self.request_verification_tokens = []

        # Lets try to parse csrf_token from homepage html head meta[name="csrf_token"]
        response = self.session.get(self.url, timeout=self.timeout)

        csrf_tokens = self.csrf_re.findall(response.content.decode('UTF-8'))
        if csrf_tokens:
            self.request_verification_tokens = csrf_tokens
        else:
            # If we did not find anything in HTML, lets try to ask API for it
            token = self._get_token()
            if token is not None:
                self.request_verification_tokens.append(token)

    def _build_final_url(self, endpoint: str, prefix: str='api') -> str:
        return urllib.parse.urljoin(self.url + '{}/'.format(prefix), endpoint)

    def post_get(self,
                 endpoint: str,
                 data: Union[dict, list, None]=None,
                 refresh_csrf: bool=False,
                 prefix: str='api',
                 dicttoxml_xargs: Optional[dict]=None) \
                 -> GetResponseType:
        return cast(
            GetResponseType,
            self._post(endpoint, data, refresh_csrf, prefix, dicttoxml_xargs)
        )

    def post_set(self,
                 endpoint: str,
                 data: Union[dict, list, None]=None,
                 refresh_csrf: bool=False,
                 prefix: str='api',
                 dicttoxml_xargs: Optional[dict]=None) \
                 -> SetResponseType:
        return cast(
            SetResponseType,
            self._post(endpoint, data, refresh_csrf, prefix, dicttoxml_xargs)
        )

    @_try_or_reload_and_retry
    def _post(self,
              endpoint: str,
              data: Union[dict, list, None]=None,
              refresh_csrf: bool=False,
              prefix: str='api',
              dicttoxml_xargs: Optional[dict]=None) \
              -> Union[GetResponseType, SetResponseType]:

        headers = {
            'Content-Type': 'application/xml'
        }
        if self.request_verification_tokens:
            if len(self.request_verification_tokens) > 1:
                headers['__RequestVerificationToken'] = self.request_verification_tokens.pop(0)
            else:
                headers['__RequestVerificationToken'] = self.request_verification_tokens[0]

        response = self.session.post(
            self._build_final_url(endpoint, prefix),
            data=self._create_request_xml(data, dicttoxml_xargs) if data else b'',
            headers=headers,
            timeout=self.timeout,
        )
        response.raise_for_status()

        response_data = cast(str, self._check_response_status(self._process_response_xml(response)))

        if refresh_csrf:
            self.request_verification_tokens = []

        if '__RequestVerificationTokenone' in response.headers:
            self.request_verification_tokens.append(response.headers['__RequestVerificationTokenone'])
            if '__RequestVerificationTokentwo' in response.headers:
                self.request_verification_tokens.append(response.headers['__RequestVerificationTokentwo'])
        elif '__RequestVerificationToken' in response.headers:
            self.request_verification_tokens.append(response.headers['__RequestVerificationToken'])
        else:
            _LOGGER.debug('Failed to get CSRF from POST response headers')

        return response_data

    def post_file(self,
                  endpoint: str,
                  files: dict,
                  data: Optional[dict]=None,
                  prefix: str = 'api',
                  ) -> str:

        if self.request_verification_tokens:
            if data is None:
                data = {}
            data['csrf_token'] = self.request_verification_tokens[0]

        response = self.session.post(
            self._build_final_url(endpoint, prefix),
            files=files,
            data=data,
            timeout=self.timeout
        )
        response.raise_for_status()

        return response.content.decode('UTF-8').lower()

    @_try_or_reload_and_retry
    def get(self, endpoint: str, params: Optional[dict]=None, prefix: str='api') -> dict:
        headers = {}
        if len(self.request_verification_tokens) == 1:
            headers['__RequestVerificationToken'] = self.request_verification_tokens[0]

        response = self.session.get(
            self._build_final_url(endpoint, prefix),
            params=params,
            headers=headers,
            timeout=self.timeout,
        )

        return cast(dict, self._check_response_status(self._process_response_xml(response)))

    def _get_token(self) -> Optional[str]:
        try:
            data = self.get('webserver/token')
            return data['token']
        except ResponseErrorNotSupportedException:
            try:
                data = self.get('webserver/SesTokInfo')
                return data['TokInfo']
            except ResponseErrorNotSupportedException:
                return None



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Bluetooth(ApiGroup):
    def settings(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._connection.get('bluetooth/settings')

    def scan(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._connection.get('bluetooth/scan')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class UsbPrinter(ApiGroup):

    def printerlist(self) -> GetResponseType:
        return self._connection.get('usbprinter/printerlist')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Ota(ApiGroup):
    def status(self) -> GetResponseType:
        return self._connection.get('ota/status')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Global(ApiGroup):
    def languagelist(self) -> GetResponseType:
        return self._connection.get('global/languagelist.xml', prefix='config')

    def config(self) -> GetResponseType:
        return self._connection.get('global/config.xml', prefix='config')

    def net_type(self) -> GetResponseType:
        return self._connection.get('global/net-type.xml', prefix='config')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Ussd(ApiGroup):
    def prepaidussd(self) -> GetResponseType:
        return self._connection.get('ussd/prepaidussd.xml', prefix='config')

    def postpaidussd(self) -> GetResponseType:
        return self._connection.get('ussd/postpaidussd.xml', prefix='config')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Wifi(ApiGroup):
    def config(self) -> GetResponseType:
        return self._connection.get('wifi/config.xml', prefix='config')

    def configure(self) -> GetResponseType:
        return self._connection.get('wifi/configure.xml', prefix='config')

    def country_channel(self) -> GetResponseType:
        return self._connection.get('wifi/countryChannel.xml', prefix='config')

    def channel_auto_match_hardware(self) -> GetResponseType:
        return self._connection.get('wifi/channelAutoMatchHardware.xml', prefix='config')



from collections import OrderedDict
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType, SetResponseType
from huawei_lte_api.enums.wlan import AuthModeEnum, WepEncryptModeEnum, WpaEncryptModeEnum


class WLan(ApiGroup):
    def wifi_feature_switch(self) -> GetResponseType:
        return self._connection.get('wlan/wifi-feature-switch')

    def station_information(self) -> GetResponseType:
        return self._connection.get('wlan/station-information')

    def basic_settings(self) -> GetResponseType:
        return self._connection.get('wlan/basic-settings')

    def set_basic_settings(self, ssid: str, hide: bool=False, wifi_restart: bool=False) -> SetResponseType:
        return self._connection.post_set('wlan/basic-settings', OrderedDict((
            ('WifiSsid', ssid),
            ('WifiHide', hide),
            ('WifiRestart', int(wifi_restart))
        )))

    def security_settings(self) -> GetResponseType:
        return self._connection.get('wlan/security-settings')

    def set_security_settings(self,
                              wpa_psk: str,
                              wep_key: str='',
                              wpa_encryption_mode: WpaEncryptModeEnum=WpaEncryptModeEnum.MIX,
                              wep_encryption_mode: WepEncryptModeEnum=WepEncryptModeEnum.WEP128,
                              auth_mode: AuthModeEnum=AuthModeEnum.AUTO,
                              wifi_restart: bool=True
                              ) -> SetResponseType:
        return self._connection.post_set('wlan/security-settings', OrderedDict((
            ('WifiAuthmode', auth_mode.value),
            ('WifiWepKey1', wep_key),
            ('WifiWpaencryptionmodes', wpa_encryption_mode.value),
            ('WifiBasicencryptionmodes', wep_encryption_mode.value),
            ('WifiWpapsk', wpa_psk),
            ('WifiRestart', int(wifi_restart))
        )))

    def multi_security_settings(self) -> GetResponseType:
        return self._connection.get('wlan/multi-security-settings')

    def multi_security_settings_ex(self) -> GetResponseType:
        return self._connection.get('wlan/multi-security-settings-ex')

    def multi_basic_settings(self) -> GetResponseType:
        return self._connection.get('wlan/multi-basic-settings')

    def set_multi_basic_settings(self, clients: list) -> SetResponseType:
        """

   :param clients: list of dicts with format {'wifihostname': hostname,'WifiMacFilterMac': mac}
   """
        return self._connection.post_set('wlan/multi-basic-settings', {
            'Ssids': {
                'Ssid': clients
            },
            'WifiRestart': 1
        })

    def host_list(self) -> GetResponseType:
        # Make sure Hosts->Host is a list
        # It may be returned as a single dict if only one is associated,
        # as well as sometimes None.
        hosts = self._connection.get('wlan/host-list')
        if hosts.get('Hosts') is None:
            hosts['Hosts'] = {}
        host = hosts['Hosts'].setdefault('Host', [])
        if isinstance(host, dict):
            hosts['Hosts']['Host'] = [host]
        return hosts

    def handover_setting(self) -> GetResponseType:
        return self._connection.get('wlan/handover-setting')

    def set_handover_setting(self, handover: int) -> SetResponseType:
        """
    G3_PREFER = 0
    WIFI_PREFER = 2
    :param handover:
    """
        return self._connection.post_set('wlan/handover-setting', {
            'Handover': handover
        })

    def multi_switch_settings(self) -> GetResponseType:
        return self._connection.get('wlan/multi-switch-settings')

    def multi_macfilter_settings(self) -> GetResponseType:
        return self._connection.get('wlan/multi-macfilter-settings')

    def set_multi_macfilter_settings(self, clients: list) -> SetResponseType:
        """

        :param clients: list of dicts with format {'wifihostname': hostname,'WifiMacFilterMac': mac}
        """
        return self._connection.post_set('wlan/multi-macfilter-settings', {
            'Ssids': {
                'Ssid': clients
            }
        })

    def multi_macfilter_settings_ex(self) -> GetResponseType:
        return self._connection.get('wlan/multi-macfilter-settings-ex')

    def mac_filter(self) -> GetResponseType:
        return self._connection.get('wlan/mac-filter')

    def set_mac_filter(self, hostname: str, mac: str) -> SetResponseType:
        return self._connection.post_set('wlan/mac-filter', OrderedDict((
            ('wifihostname', hostname),
            ('WifiMacFilterMac', mac)
        )))

    def oled_showpassword(self) -> GetResponseType:
        return self._connection.get('wlan/oled-showpassword')

    def wps(self) -> GetResponseType:
        return self._connection.get('wlan/wps')

    def wps_appin(self) -> GetResponseType:
        return self._connection.get('wlan/wps-appin')

    def wps_pbc(self) -> GetResponseType:
        return self._connection.get('wlan/wps-pbc')

    def wps_switch(self) -> GetResponseType:
        return self._connection.get('wlan/wps-switch')

    def status_switch_settings(self) -> GetResponseType:
        return self._connection.get('wlan/status-switch-settings')

    def wifiprofile(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage, probably not implemented by Huawei
        :return:
        """
        return self._connection.get('wlan/wifiprofile')

    def wififrequence(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage, probably not implemented by Huawei
        :return:
        """
        return self._connection.get('wlan/wififrequence')

    def wifiscanresult(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage, probably not implemented by Huawei
        :return:
        """
        return self._connection.get('wlan/wifiscanresult')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Dhcp(ApiGroup):
    def settings(self) -> GetResponseType:
        return self._connection.get('dhcp/settings')

    def feature_switch(self) -> GetResponseType:
        return self._connection.get('dhcp/feature-switch')

    def dhcp_host_info(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._connection.get('dhcp/dhcp-host-info')

    def static_addr_info(self) -> GetResponseType:
        """
        Endpoint found by reverse engineering B310s-22 firmware, unknown usage
        :return:
        """
        return self._connection.get('dhcp/static-addr-info')



#!/usr/bin/env python3

from argparse import ArgumentParser
import os.path
import pprint
import sys
from typing import Any, Callable
sys.path.insert(0, os.path.join(os.path.dirname(__file__), os.path.pardir))

from huawei_lte_api.AuthorizedConnection import AuthorizedConnection
from huawei_lte_api.Client import Client
from huawei_lte_api.Connection import Connection


parser = ArgumentParser()
parser.add_argument('url', type=str)
parser.add_argument('--username', type=str)
parser.add_argument('--password', type=str)
args = parser.parse_args()

if args.username is None:
    connection = Connection(args.url)
else:
    connection = AuthorizedConnection(
        args.url, username=args.username, password=args.password)

client = Client(connection)

def dump(method: Callable[[], Any]) -> None:
    print("==== %s" % method)
    try:
        pprint.pprint(method())
    except Exception as e:
        print(str(e))
    print("")

dump(client.device.information)
dump(client.device.autorun_version)
dump(client.device.device_feature_switch)
dump(client.device.basic_information)
dump(client.device.basicinformation)
dump(client.device.usb_tethering_switch)
dump(client.device.boot_time)
dump(client.device.signal)
dump(client.device.antenna_status)
dump(client.device.antenna_type)
dump(client.device.logsetting)

dump(client.user.state_login)
dump(client.user.remind)
dump(client.user.password)
dump(client.user.pwd)
dump(client.user.authentication_login)
dump(client.user.challenge_login)
dump(client.user.hilink_login)
dump(client.user.history_login)
dump(client.user.heartbeat)
dump(client.user.web_feature_switch)
dump(client.user.screen_state)
dump(client.user.session)

dump(client.monitoring.converged_status)
dump(client.monitoring.status)
dump(client.monitoring.check_notifications)
dump(client.monitoring.traffic_statistics)
dump(client.monitoring.start_date)
dump(client.monitoring.start_date_wlan)
dump(client.monitoring.month_statistics)
dump(client.monitoring.month_statistics_wlan)
dump(client.monitoring.wifi_month_setting)

dump(client.security.bridgemode)
dump(client.security.get_firewall_switch)
dump(client.security.mac_filter)
dump(client.security.lan_ip_filter)
dump(client.security.virtual_servers)
dump(client.security.url_filter)
dump(client.security.upnp)
dump(client.security.dmz)
dump(client.security.sip)
dump(client.security.feature_switch)
dump(client.security.nat)
dump(client.security.special_applications)
dump(client.security.white_lan_ip_filter)
dump(client.security.white_url_filter)
dump(client.security.acls)

dump(client.webserver.publickey)
dump(client.webserver.token)
dump(client.webserver.white_list_switch)

dump(client.global_.module_switch)

dump(client.wlan.wifi_feature_switch)
dump(client.wlan.station_information)
dump(client.wlan.basic_settings)
dump(client.wlan.security_settings)
dump(client.wlan.multi_security_settings)
dump(client.wlan.multi_security_settings_ex)
dump(client.wlan.multi_basic_settings)
dump(client.wlan.host_list)
dump(client.wlan.handover_setting)
dump(client.wlan.multi_switch_settings)
dump(client.wlan.multi_macfilter_settings)
dump(client.wlan.multi_macfilter_settings_ex)
dump(client.wlan.mac_filter)
dump(client.wlan.oled_showpassword)
dump(client.wlan.wps)
dump(client.wlan.wps_appin)
dump(client.wlan.wps_pbc)
dump(client.wlan.wps_switch)
dump(client.wlan.status_switch_settings)
dump(client.wlan.wifiprofile)
dump(client.wlan.wififrequence)
dump(client.wlan.wifiscanresult)

dump(client.cradle.status_info)
dump(client.cradle.feature_switch)
dump(client.cradle.basic_info)
dump(client.cradle.factory_mac)
dump(client.cradle.mac_info)

dump(client.pin.status)
dump(client.pin.simlock)
dump(client.pin.save_pin)

dump(client.language.current_language)

dump(client.config_device_information.config)

dump(client.config_dialup.config)
dump(client.config_dialup.connectmode)
dump(client.config_dialup.profileswitch)
dump(client.config_dialup.lmt_auto_mode_disconnect)

dump(client.config_global.languagelist)
dump(client.config_global.config)
dump(client.config_global.net_type)

dump(client.config_lan.config)

dump(client.config_network.config)
dump(client.config_network.net_mode)
dump(client.config_network.networkmode)
dump(client.config_network.networkband_null)

dump(client.config_pc_assistant.config)
dump(client.config_pc_assistant.updateautorun)

dump(client.config_pincode.config)

dump(client.config_sms.config)

dump(client.config_voice.config)

dump(client.config_web_ui_cfg.config)

dump(client.config_wifi.configure)
dump(client.config_wifi.country_channel)
dump(client.config_wifi.channel_auto_match_hardware)

dump(client.config_device.config)

dump(client.config_fast_boot.config)

dump(client.config_firewall.config)

dump(client.config_ipv6.config)

dump(client.config_ota.config)

dump(client.config_pb.config)

dump(client.config_sntp.config)

dump(client.config_statistic.config)

dump(client.config_stk.config)

dump(client.config_update.config)

dump(client.config_u_pnp.config)

dump(client.config_ussd.prepaidussd)
dump(client.config_ussd.postpaidussd)

dump(client.config_web_sd.config)

dump(client.usermanual_public_sys_resources.config)

dump(client.ota.status)

dump(client.net.current_plmn)
dump(client.net.net_mode)
dump(client.net.network)
dump(client.net.register)
dump(client.net.net_mode_list)
# DoS? dump(client.net.plmn_list)
dump(client.net.net_feature_switch)
dump(client.net.cell_info)
dump(client.net.csps_state)

dump(client.dial_up.mobile_dataswitch)
dump(client.dial_up.connection)
dump(client.dial_up.dialup_feature_switch)
dump(client.dial_up.profiles)
dump(client.dial_up.auto_apn)

dump(client.sms.get_cbsnewslist)
dump(client.sms.sms_count)
dump(client.sms.send_status)
dump(client.sms.get_sms_list)
dump(client.sms.config)
dump(client.sms.sms_count_contact)
dump(client.sms.get_sms_list_pdu)

dump(client.redirection.homepage)

dump(client.v_sim.operateswitch_vsim)

dump(client.dhcp.settings)
dump(client.dhcp.feature_switch)
dump(client.dhcp.dhcp_host_info)
dump(client.dhcp.static_addr_info)

dump(client.d_dns.get_ddns_list)
dump(client.d_dns.get_status)

dump(client.diagnosis.trace_route_result)
dump(client.diagnosis.diagnose_ping)
dump(client.diagnosis.diagnose_traceroute)
dump(client.diagnosis.time_reboot)

dump(client.s_ntp.get_settings)
dump(client.s_ntp.sntpswitch)
dump(client.s_ntp.serverinfo)
dump(client.s_ntp.timeinfo)

dump(client.online_update.check_new_version)
dump(client.online_update.status)
dump(client.online_update.url_list)
dump(client.online_update.ack_newversion)
# May cause device reboot: dump(client.online_update.cancel_downloading)
dump(client.online_update.upgrade_messagebox)
dump(client.online_update.configuration)
dump(client.online_update.autoupdate_config)
dump(client.online_update.redirect_cancel)

dump(client.log.loginfo)

dump(client.time.timeout)

dump(client.sd_card.dlna_setting)
dump(client.sd_card.sdcard)
dump(client.sd_card.sdcardsamba)
dump(client.sd_card.printerlist)
dump(client.sd_card.share_account)

dump(client.usb_storage.fsstatus)
dump(client.usb_storage.usbaccount)

dump(client.usb_printer.printerlist)

dump(client.vpn.feature_switch)
dump(client.vpn.br_list)
dump(client.vpn.ipsec_settings)
dump(client.vpn.l2tp_settings)
dump(client.vpn.pptp_settings)
dump(client.vpn.status)

dump(client.ntwk.lan_upnp_portmapping)
dump(client.ntwk.celllock)

dump(client.pb.get_pb_list)
dump(client.pb.pb_count)
dump(client.pb.group_count)

dump(client.syslog.querylog)

dump(client.voice.featureswitch)
dump(client.voice.sipaccount)
dump(client.voice.sipadvance)
dump(client.voice.sipserver)
dump(client.voice.speeddial)
dump(client.voice.functioncode)
dump(client.voice.voiceadvance)
dump(client.voice.codec)

dump(client.cwmp.basic_info)

dump(client.lan.host_info)

dump(client.led.nightmode)

dump(client.statistic.feature_roam_statistic)

dump(client.timerule.timerule)

dump(client.bluetooth.settings)
dump(client.bluetooth.scan)

dump(client.mlog.mobile_logger)

dump(client.voice.voicebusy)

if isinstance(connection, AuthorizedConnection):
    client.user.logout()



import datetime
from typing import Optional, Tuple, Union
from urllib.parse import urlparse, urlunparse
from huawei_lte_api.Connection import Connection


class AuthorizedConnection(Connection):
    LOGOUT_TIMEOUT = 300  # seconds
    login_time = None
    logged_in = False

    def __init__(self, url: str, username: Optional[str]=None, password: Optional[str]=None,
                 login_on_demand: bool=False, timeout: Union[float, Tuple[float, float], None] = None):
        # Auth info embedded in the URL may reportedly cause problems, strip it
        parsed_url = urlparse(url)
        clear_url = urlunparse((
            parsed_url.scheme,
            parsed_url.netloc.rpartition("@")[-1],
            *parsed_url[2:]
        ))
        super().__init__(clear_url, timeout=timeout)
        username = username if username else parsed_url.username
        password = password if password else parsed_url.password

        from huawei_lte_api.api.User import User  # pylint: disable=cyclic-import,import-outside-toplevel
        self.user = User(self, username, password)

        if not login_on_demand:
            if self.user.login(True):
                self.login_time = datetime.datetime.utcnow()
                self.logged_in = True

    def _is_login_timeout(self) -> bool:
        if self.login_time is None:
            return True
        logout_time = self.login_time + datetime.timedelta(seconds=self.LOGOUT_TIMEOUT)
        return logout_time < datetime.datetime.utcnow()

    def enforce_authorized_connection(self) -> bool:
        # Check if connection timeouted or not
        if not self.logged_in or self._is_login_timeout():
            # Connection timeouted, relogin
            if self.user.login():
                self.login_time = datetime.datetime.utcnow()
                self.logged_in = True
            else:
                self.login_time = None
                self.logged_in = False

        return self.logged_in



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType, SetResponseType


class DialUp(ApiGroup):
    def mobile_dataswitch(self) -> GetResponseType:
        """
        Get current LTE modem toggle state
        :return:
        """
        return self._connection.get('dialup/mobile-dataswitch')

    def connection(self) -> GetResponseType:
        return self._connection.get('dialup/connection')

    def dialup_feature_switch(self) -> GetResponseType:
        return self._connection.get('dialup/dialup-feature-switch')

    def profiles(self) -> GetResponseType:
        return self._connection.get('dialup/profiles')

    def auto_apn(self) -> GetResponseType:
        return self._connection.get('dialup/auto-apn')

    def dial(self) -> SetResponseType:
        return self._connection.post_set('dialup/dial', {
            'Action': 1
        })

    def set_mobile_dataswitch(self, dataswitch: int = 0) -> SetResponseType:
        """
        Toggle LTE modem state
        :param dataswitch: 0 to disable LTE modem, 1 to enable LTE modem
        """
        return self._connection.post_set('dialup/mobile-dataswitch', {
            'dataswitch': dataswitch
        })



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Cwmp(ApiGroup):
    def basic_info(self) -> GetResponseType:
        return self._connection.get('cwmp/basic-info')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class UPnp(ApiGroup):
    def config(self) -> GetResponseType:
        return self._connection.get('upnp/config.xml', prefix='config')



class ResponseErrorException(Exception):
    def __init__(self, message: str, code: int) -> None:
        super().__init__(message)
        self.code = code


class ResponseErrorNotSupportedException(ResponseErrorException):
    pass


class ResponseErrorLoginRequiredException(ResponseErrorException):
    pass


class ResponseErrorSystemBusyException(ResponseErrorException):
    pass


class ResponseErrorLoginCsfrException(ResponseErrorException):
    """Deprecated misspelling, use ResponseErrorLoginCsrfException instead."""


class ResponseErrorLoginCsrfException(ResponseErrorLoginCsfrException):
    pass


class LoginErrorUsernameWrongException(ResponseErrorException):
    pass


class LoginErrorPasswordWrongException(ResponseErrorException):
    pass


class LoginErrorAlreadyLoginException(ResponseErrorException):
    pass


class LoginErrorUsernamePasswordWrongException(ResponseErrorException):
    pass


class LoginErrorUsernamePasswordOverrunException(ResponseErrorException):
    pass


class LoginErrorUsernamePasswordModifyException(ResponseErrorException):
    pass



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType, SetResponseType


class Language(ApiGroup):
    def set_current_language(self, current_language: str) -> SetResponseType:
        return self._connection.post_set('language/current-language', {
            'CurrentLanguage': current_language
        })

    def current_language(self) -> GetResponseType:
        return self._connection.get('language/current-language')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Lan(ApiGroup):
    def host_info(self) -> GetResponseType:
        return self._connection.get('lan/HostInfo')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Network(ApiGroup):
    def net_mode(self) -> GetResponseType:
        return self._connection.get('network/net-mode.xml', prefix='config')

    def networkmode(self) -> GetResponseType:
        return self._connection.get('network/networkmode.xml', prefix='config')

    def config(self) -> GetResponseType:
        return self._connection.get('network/config.xml', prefix='config')

    def networkband_null(self) -> GetResponseType:
        return self._connection.get('network/networkband_null.xml', prefix='config')

    def set_only_4g(self) -> GetResponseType:
        return self._connection.get('network/setOnly4g.xml', prefix='config')



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.Connection import GetResponseType


class Statistic(ApiGroup):
    def feature_roam_statistic(self) -> GetResponseType:
        return self._connection.get('statistic/feature-roam-statistic')



import enum


@enum.unique
class ConnectionStatusEnum(enum.IntEnum):
    CONNECTING = 900
    CONNECTED = 901
    DISCONNECTED = 902
    DISCONNECTING = 903
    CONNECT_FAILED = 904
    CONNECT_STATUS_NULL = 905
    CONNECT_STATUS_ERROR = 906
